/*! \file 
 * X-Forge Core <br>
 * Copyright 2000-2003 Fathammer Ltd
 * 
 * \brief Vector math header file
 * 
 * $Id: XFcVector.h,v 1.20 2003/03/10 09:53:23 jari Exp $
 * $Date: 2003/03/10 09:53:23 $
 * $Revision: 1.20 $
 */

#ifndef XFC_MATH_VECTOR_H_INCLUDED
#define XFC_MATH_VECTOR_H_INCLUDED


#include <xfcore/XFcFixed.h>

#ifdef XFC_USE_INTELGPP
#include <external/internal/gpp.h>
#endif


// disable warning C4514: unreferenced inline function has been removed
#ifdef _WIN32
#pragma warning ( disable : 4514 )
#endif


//! 4-Dimensional vector container.
/*!
 * \sa XFcMath
 */
class XFcVector4 
{
public:
    REAL x, y, z, w;

    XFcVector4() {}
    XFcVector4(REAL aX, REAL aY, REAL aZ, REAL aW) : x(aX), y(aY), z(aZ), w(aW) {}
};


//! 3-Dimensional vector.
/*! Typical vector operators implemented.
 * \sa XFcMath
 */
class XFcVector3 
{
public:

    //! The vector components.
    REAL x, y, z;

    //! Empty constructor.
    XFcVector3() {}
    //! Creation from 3 REALs.
    XFcVector3(REAL aX, REAL aY, REAL aZ) : x(aX), y(aY), z(aZ) {}

    //! Adds components of another vector to this vector.
    /*! \param aVec vector to be added to this vector.
     * \return reference to this vector.
     */
    inline XFcVector3 & operator+=(const XFcVector3 &aVec);
    //! Adds components of another vector to this vector.
    /*! \param aVec vector to be added to this vector.
     * \return a new result vector.
     */
    inline XFcVector3 operator+(const XFcVector3 &aVec) const;
    //! Adds an integer value to each component of this vector.
    /*! \param aArg integer value for the addition.
     * \return a new result vector.
     */
    inline XFcVector3 operator+(const INT32 aArg) const;
    //! Adds a floating point value to each component of this vector.
    /*! \param aArg floating point value for the addition.
     * \return a new result vector.
     */
    inline XFcVector3 operator+(const FLOAT32 aArg) const;

    //! Subtracts components of another vector from this vector.
    /*! \param aVec vector to be subtracted from this vector.
     * \return reference to this vector.
     */
    inline XFcVector3 & operator-=(const XFcVector3 &aVec);
    //! Subtracts components of another vector from this vector.
    /*! \param aVec vector to be subtracted from this vector.
     * \return a new result vector.
     */
    inline XFcVector3 operator-(const XFcVector3 &aVec) const;
    //! Subtracts an integer value from each component of this vector.
    /*! \param aArg integer value for the subtraction.
     * \return a new result vector.
     */
    inline XFcVector3 operator-(const INT32 aArg) const;
    //! Subtracts a floating point value from each component of this vector.
    /*! \param aArg floating point value for the subtraction.
     * \return a new result vector.
     */
    inline XFcVector3 operator-(const FLOAT32 aArg) const;

    //! Multiplies components of this vector with an integer value.
    /*! \param aArg integer value for the multiplication.
     * \return reference to this vector.
     */
    inline XFcVector3 & operator*=(const INT32 aArg);
    //! Multiplies components of this vector with a floating point value.
    /*! \param aArg floating point value for the multiplication.
     * \return reference to this vector.
     */
    inline XFcVector3 & operator*=(const FLOAT32 aArg);

    //! Multiplies components of this vector with an integer value.
    /*! \param aArg integer value for the multiplication.
     * \return a new result vector.
     */
    inline XFcVector3 operator*(const INT32 aArg) const;
    //! Multiplies components of this vector with a floating point value.
    /*! \param aArg floating point value for the multiplication.
     * \return a new result vector.
     */
    inline XFcVector3 operator*(const FLOAT32 aArg) const;
    //! Multiplies components of a vector with an integer value.
    /*! \param aArg integer value for the multiplication.
     *  \param aVec vector to be multiplied.
     * \return a new result vector.
     */
    friend XFcVector3 operator*(const INT32 aArg, const XFcVector3 &aVec);
    //! Multiplies components of a vector with a floating point value.
    /*! \param aArg floating point value for the multiplication.
     *  \param aVec vector to be multiplied.
     * \return a new result vector.
     */
    friend XFcVector3 operator*(const FLOAT32 aArg, const XFcVector3 &aVec);

    //! Divides components of this vector with an integer value.
    /*! \param aArg integer value for the division.
     * \return reference to this vector.
     */
    inline XFcVector3 & operator/=(const INT32 aArg);
    //! Divides components of this vector with a floating point value.
    /*! \param aArg floating point value for the division.
     * \return reference to this vector.
     */
    inline XFcVector3 & operator/=(const FLOAT32 aArg);

    //! Divides components of this vector with an integer value.
    /*! \param aArg integer value for the division.
     * \return a new result vector.
     */
    inline XFcVector3 operator/(const INT32 aArg) const;
    //! Divides components of this vector with a floating point value.
    /*! \param aArg floating point value for the division.
     * \return a new result vector.
     */
    inline XFcVector3 operator/(const FLOAT32 aArg) const;
    //! Divides components of a vector with an integer value.
    /*! \param aArg integer value for the division.
     *  \param aVec vector to be divided.
     * \return a new result vector.
     */
    friend XFcVector3 operator/(const INT32 aArg, const XFcVector3 &aVec);
    //! Divides components of a vector with a floating point value.
    /*! \param aArg floating point value for the division.
     *  \param aVec vector to be divided.
     * \return a new result vector.
     */
    friend XFcVector3 operator/(const FLOAT32 aArg, const XFcVector3 &aVec);

#ifndef XFC_NO_REAL_OPERATORS

    //! Adds a REAL value to each component of this vector.
    /*! \param aArg value for the addition.
     * \return a new result vector.
     */
    inline XFcVector3 operator+(const REAL &aArg) const; 
    //! Subtracts a REAL value from each component of this vector.
    /*! \param aArg value for the subtraction.
     * \return a new result vector.
     */
    inline XFcVector3 operator-(const REAL &aArg) const; 
    //! Multiplies components of this vector with a REAL value.
    /*! \param aArg value for the multiplication.
     * \return reference to this vector.
     */
    inline XFcVector3 & operator*=(const REAL &aArg);
    //! Multiplies components of this vector with a REAL value.
    /*! \param aArg value for the multiplication.
     * \return a new result vector.
     */
    inline XFcVector3 operator*(const REAL &aArg) const;
    //! Multiplies components of a vector with a REAL value.
    /*! \param aArg value for the multiplication.
     *  \param aVec vector to be multiplied.
     * \return a new result vector.
     */
    friend XFcVector3 operator*(const REAL &aArg, const XFcVector3 &aVec);
    //! Divides components of this vector with a REAL value.
    /*! \param aArg value for the division.
     * \return reference to this vector.
     */
    inline XFcVector3 & operator/=(const REAL &aArg);
    //! Divides components of this vector with a REAL value.
    /*! \param aArg integer value for the division.
     * \return a new result vector.
     */
    inline XFcVector3 operator/(const REAL &aArg) const;
    //! Divides components of a vector with a REAL value.
    /*! \param aArg value for the division.
     *  \param aVec vector to be divided.
     * \return a new result vector.
     */
    friend XFcVector3 operator/(const REAL &aArg, const XFcVector3 &aVec);

#endif // !XFC_NO_REAL_OPERATORS

    //! Negates all components of this vector.
    /*! \return a new result vector.
     */
    inline XFcVector3 operator - () const { return XFcVector3(-x, -y, -z); }

    //! Returns dot product of this vector with an another vector.
    /*! \param aVec the other vector of the dot product.
     * \return dot product of the vectors.
     */
    inline REAL dotProduct(const XFcVector3 &aVec);
    //! Returns cross product of this vector with an another vector.
    /*! \param aVec the other vector of the cross product.
     * \return a new result vector which is the cross product of the vectors.
     */
    inline XFcVector3 crossProduct(const XFcVector3 &aVec);
    //! Returns length of this vector.
    /*! \sa lengthSq
     */
    inline REAL length();
    //! Returns squared length of this vector.
    /*! \note Calculating squared length is faster than calculating length.
     * \sa length
     */
    inline REAL lengthSq();

    //! Normalizes this vector.
    inline void normalize();
};


XFcVector3 & XFcVector3::operator+=(const XFcVector3 &aVec)
{
    x += aVec.x;
    y += aVec.y;
    z += aVec.z;
    return *this;
}


inline XFcVector3 XFcVector3::operator+(const XFcVector3 &aVec) const
{
    return XFcVector3(x + aVec.x, y + aVec.y, z + aVec.z);
}


inline XFcVector3 XFcVector3::operator+(const INT32 aArg) const
{
    return XFcVector3(x + aArg, y + aArg, z + aArg);
}


inline XFcVector3 XFcVector3::operator+(const FLOAT32 aArg) const
{
    return XFcVector3(x + aArg, y + aArg, z + aArg);
}


XFcVector3 & XFcVector3::operator-=(const XFcVector3 &aVec)
{
    x -= aVec.x;
    y -= aVec.y;
    z -= aVec.z;
    return *this;
}


inline XFcVector3 XFcVector3::operator-(const XFcVector3 &aVec) const
{
    return XFcVector3(x - aVec.x, y - aVec.y, z - aVec.z);
}


inline XFcVector3 XFcVector3::operator-(const INT32 aArg) const
{
    return XFcVector3(x - aArg, y - aArg, z - aArg);
}


inline XFcVector3 XFcVector3::operator-(const FLOAT32 aArg) const
{
    return XFcVector3(x - aArg, y - aArg, z - aArg);
}


inline XFcVector3 & XFcVector3::operator*=(const INT32 aArg)
{
    x *= aArg;
    y *= aArg;
    z *= aArg;
    return *this;
}


inline XFcVector3 & XFcVector3::operator*=(const FLOAT32 aArg)
{
    x *= aArg;
    y *= aArg;
    z *= aArg;
    return *this;
}


inline XFcVector3 & XFcVector3::operator/=(const INT32 aArg)
{
    x /= aArg;
    y /= aArg;
    z /= aArg;
    return *this;
}


inline XFcVector3 & XFcVector3::operator/=(const FLOAT32 aArg)
{
    x /= aArg;
    y /= aArg;
    z /= aArg;
    return *this;
}


#ifndef XFC_NO_REAL_OPERATORS

inline XFcVector3 XFcVector3::operator+(const REAL &aArg) const
{
    return XFcVector3(x + aArg, y + aArg, z + aArg);
}

inline XFcVector3 XFcVector3::operator-(const REAL &aArg) const
{
    return XFcVector3(x - aArg, y - aArg, z - aArg);
}

inline XFcVector3 & XFcVector3::operator*=(const REAL &aArg)
{
    x *= aArg;
    y *= aArg;
    z *= aArg;
    return *this;
}

inline XFcVector3 XFcVector3::operator*(const REAL &aArg) const
{
    return XFcVector3(x * aArg, y * aArg, z * aArg);
}

inline XFcVector3 & XFcVector3::operator/=(const REAL &aArg)
{
    x /= aArg;
    y /= aArg;
    z /= aArg;
    return *this;
}

inline XFcVector3 XFcVector3::operator/(const REAL &aArg) const
{
    return XFcVector3(x / aArg, y / aArg, z / aArg);
}

inline XFcVector3 operator*(const REAL &aArg, const XFcVector3 &aVec)
{
    return aVec * aArg;
}

#endif //XFC_NO_REAL_OPERATORS

inline XFcVector3 XFcVector3::operator/(const FLOAT32 aArg) const
{
    return XFcVector3(x / aArg, y / aArg, z / aArg);
}


inline XFcVector3 XFcVector3::operator*(const INT32 aArg) const
{
    return XFcVector3(x * aArg, y * aArg, z * aArg);
}


inline XFcVector3 XFcVector3::operator*(const FLOAT32 aArg) const
{
    return XFcVector3(x * aArg, y * aArg, z * aArg);
}


inline XFcVector3 operator*(const INT32 aArg, const XFcVector3 &aVec)
{
    return aVec * aArg;
}


inline XFcVector3 operator*(const FLOAT32 aArg, const XFcVector3 &aVec)
{
    return aVec * aArg;
}


inline REAL XFcVector3::dotProduct(const XFcVector3 &aVec)
{
#ifdef XFC_USE_INTELGPP
    XFcFixed res;
    gppVec3DDot_16_32s((GPP_VEC3D *)this, (GPP_VEC3D *)&aVec, (I32 *)&res.mValue);
    return res;
#else
    return x * aVec.x + y * aVec.y + z * aVec.z;
#endif
}


inline XFcVector3 XFcVector3::crossProduct(const XFcVector3 &aVec)
{
#ifdef XFC_USE_INTELGPP
    XFcVector3 res;
    gppVec3DCross_16_32s((GPP_VEC3D *)this, (GPP_VEC3D *)&aVec, (GPP_VEC3D *)&res);
    return res;
#else
    REAL a, b, c;
    a = y * aVec.z - z * aVec.y;
    b = -(x * aVec.z - z * aVec.x);
    c = x * aVec.y - y * aVec.x;
    /*float a,b,c;
      a = (float)y * (float)aVec.z - (float)z * (float)aVec.y;
      b = -((float)x * (float)aVec.z - (float)z * (float)aVec.x);
      c = (float)x * (float)aVec.y - (float)y * (float)aVec.x;*/
    return XFcVector3(REAL(a), REAL(b), REAL(c));
#endif
}


inline REAL XFcVector3::length()
{
#ifdef XFC_USE_INTELGPP
    XFcFixed res;
    gppVec3DLength_16_32s((GPP_VEC3D *)this, (U32 *)&res.mValue);
    return res;
#else
    //return (REAL)sqrt(lengthSq());
    return XFcMath::sqrt(lengthSq());
#endif
}


inline REAL XFcVector3::lengthSq()
{
#ifdef XFC_USE_INTELGPP
    XFcFixed res;
    gppVec3DLengthSq_16_32s((GPP_VEC3D *)this, (U32 *)&res.mValue);
    return res;
#else
    //float temp = (float)(x) * (float)(x) + (float)(y) * (float)(y) + (float)(z) * (float)(z);
    REAL temp = x * x + y * y + z * z;
    return REAL(temp);
#endif
}

inline void XFcVector3::normalize()
{
#ifdef XFC_USE_INTELGPP
    gppVec3DNormalize_16_32s((GPP_VEC3D *)this);
#else
    REAL len = length();
    if (len != REALi(0))
        *this /= len;
#endif
/*
    float length = (float)sqrt(float(x) * float(x) +
                               float(y) * float(y) +
                               float(z) * float(z));
    if (length != 0)
    {
        *this /= length;
    }
*/
}


//! 2-Dimensional vector.
/*! Typical vector operators implemented.
 * \sa XFcVector3
 */
class XFcVector2 
{
public:
    REAL x, y;

    XFcVector2() {}
    XFcVector2(REAL aX, REAL aY) : x(aX), y(aY) {}

    //! Adds components of another vector to this vector.
    /*! \param aVec vector to be added to this vector.
     * \return reference to this vector.
     */
    inline XFcVector2 & operator+=(const XFcVector2 &aVec);
    //! Adds components of another vector to this vector.
    /*! \param aVec vector to be added to this vector.
     * \return a new result vector.
     */
    inline XFcVector2 operator+(const XFcVector2 &aVec) const;
    //! Adds an integer value to each component of this vector.
    /*! \param aArg integer value for the addition.
     * \return a new result vector.
     */
    inline XFcVector2 operator+(const INT32 aArg) const;
    //! Adds a floating point value to each component of this vector.
    /*! \param aArg floating point value for the addition.
     * \return a new result vector.
     */
    inline XFcVector2 operator+(const FLOAT32 aArg) const;

    //! Subtracts components of another vector from this vector.
    /*! \param aVec vector to be subtracted from this vector.
     * \return reference to this vector.
     */
    inline XFcVector2 & operator-=(const XFcVector2 &aVec);
    //! Subtracts components of another vector from this vector.
    /*! \param aVec vector to be subtracted from this vector.
     * \return a new result vector.
     */
    inline XFcVector2 operator-(const XFcVector2 &aVec) const;
    //! Subtracts an integer value from each component of this vector.
    /*! \param aArg integer value for the subtraction.
     * \return a new result vector.
     */
    inline XFcVector2 operator-(const INT32 aArg) const;
    //! Subtracts a floating point value from each component of this vector.
    /*! \param aArg floating point value for the subtraction.
     * \return a new result vector.
     */
    inline XFcVector2 operator-(const FLOAT32 aArg) const;

    //! Multiplies components of this vector with an integer value.
    /*! \param aArg integer value for the multiplication.
     * \return reference to this vector.
     */
    inline XFcVector2 & operator*=(const INT32 aArg);
    //! Multiplies components of this vector with a floating point value.
    /*! \param aArg floating point value for the multiplication.
     * \return reference to this vector.
     */
    inline XFcVector2 & operator*=(const FLOAT32 aArg);

    //! Multiplies components of this vector with an integer value.
    /*! \param aArg integer value for the multiplication.
     * \return a new result vector.
     */
    inline XFcVector2 operator*(const INT32 aArg) const;
    //! Multiplies components of this vector with a floating point value.
    /*! \param aArg floating point value for the multiplication.
     * \return a new result vector.
     */
    inline XFcVector2 operator*(const FLOAT32 aArg) const;
    //! Multiplies components of a vector with an integer value.
    /*! \param aArg integer value for the multiplication.
     *  \param aVec vector to be multiplied.
     * \return a new result vector.
     */
    friend XFcVector2 operator*(const INT32 aArg, const XFcVector2 &aVec);
    //! Multiplies components of a vector with a floating point value.
    /*! \param aArg floating point value for the multiplication.
     *  \param aVec vector to be multiplied.
     * \return a new result vector.
     */
    friend XFcVector2 operator*(const FLOAT32 aArg, const XFcVector2 &aVec);

    //! Divides components of this vector with an integer value.
    /*! \param aArg integer value for the division.
     * \return reference to this vector.
     */
    inline XFcVector2 & operator/=(const INT32 aArg);
    //! Divides components of this vector with a floating point value.
    /*! \param aArg floating point value for the division.
     * \return reference to this vector.
     */
    inline XFcVector2 & operator/=(const FLOAT32 aArg);

    //! Divides components of this vector with an integer value.
    /*! \param aArg integer value for the division.
     * \return a new result vector.
     */
    inline XFcVector2 operator/(const INT32 aArg) const;
    //! Divides components of this vector with a floating point value.
    /*! \param aArg floating point value for the division.
     * \return a new result vector.
     */
    inline XFcVector2 operator/(const FLOAT32 aArg) const;
    //! Divides components of a vector with an integer value.
    /*! \param aArg integer value for the division.
     *  \param aVec vector to be divided.
     * \return a new result vector.
     */
    friend XFcVector2 operator/(const INT32 aArg, const XFcVector2 &aVec);
    //! Divides components of a vector with a floating point value.
    /*! \param aArg floating point value for the division.
     *  \param aVec vector to be divided.
     * \return a new result vector.
     */
    friend XFcVector2 operator/(const FLOAT32 aArg, const XFcVector2 &aVec);

#ifndef XFC_NO_REAL_OPERATORS

    //! Adds a REAL value to each component of this vector.
    /*! \param aArg value for the addition.
     * \return a new result vector.
     */
    inline XFcVector2 operator+(const REAL &aArg) const; 
    //! Subtracts a REAL value from each component of this vector.
    /*! \param aArg value for the subtraction.
     * \return a new result vector.
     */
    inline XFcVector2 operator-(const REAL &aArg) const; 
    //! Multiplies components of this vector with a REAL value.
    /*! \param aArg value for the multiplication.
     * \return reference to this vector.
     */
    inline XFcVector2 & operator*=(const REAL &aArg);
    //! Multiplies components of this vector with a REAL value.
    /*! \param aArg value for the multiplication.
     * \return a new result vector.
     */
    inline XFcVector2 operator*(const REAL &aArg) const;
    //! Multiplies components of a vector with a REAL value.
    /*! \param aArg value for the multiplication.
     *  \param aVec vector to be multiplied.
     * \return a new result vector.
     */
    friend XFcVector2 operator*(const REAL &aArg, const XFcVector2 &aVec);
    //! Divides components of this vector with a REAL value.
    /*! \param aArg value for the division.
     * \return reference to this vector.
     */
    inline XFcVector2 & operator/=(const REAL &aArg);
    //! Divides components of this vector with a REAL value.
    /*! \param aArg integer value for the division.
     * \return a new result vector.
     */
    inline XFcVector2 operator/(const REAL &aArg) const;
    //! Divides components of a vector with a REAL value.
    /*! \param aArg value for the division.
     *  \param aVec vector to be divided.
     * \return a new result vector.
     */
    friend XFcVector2 operator/(const REAL &aArg, const XFcVector2 &aVec);

#endif // !XFC_NO_REAL_OPERATORS

    //! Negates all components of this vector.
    /*! \return a new result vector.
     */
    inline XFcVector2 operator - () const { return XFcVector2(-x, -y); }

    //! Returns dot product of this vector with an another vector.
    /*! \param aVec the other vector of the dot product.
     * \return dot product of the vectors.
     */
    inline REAL dotProduct(const XFcVector2 &aVec);
    //! Returns length of this vector.
    /*! \sa lengthSq
     */
    inline REAL length();
    //! Returns squared length of this vector.
    /*! \note Calculating squared length is faster than calculating length.
     * \sa length
     */
    inline REAL lengthSq();

    //! Normalizes this vector.
    inline void normalize();
};


XFcVector2 & XFcVector2::operator+=(const XFcVector2 &aVec)
{
    x += aVec.x;
    y += aVec.y;
    return *this;
}


inline XFcVector2 XFcVector2::operator+(const XFcVector2 &aVec) const
{
    return XFcVector2(x + aVec.x, y + aVec.y);
}


inline XFcVector2 XFcVector2::operator+(const INT32 aArg) const
{
    return XFcVector2(x + aArg, y + aArg);
}


inline XFcVector2 XFcVector2::operator+(const FLOAT32 aArg) const
{
    return XFcVector2(x + aArg, y + aArg);
}


XFcVector2 & XFcVector2::operator-=(const XFcVector2 &aVec)
{
    x -= aVec.x;
    y -= aVec.y;
    return *this;
}


inline XFcVector2 XFcVector2::operator-(const XFcVector2 &aVec) const
{
    return XFcVector2(x - aVec.x, y - aVec.y);
}


inline XFcVector2 XFcVector2::operator-(const INT32 aArg) const
{
    return XFcVector2(x - aArg, y - aArg);
}


inline XFcVector2 XFcVector2::operator-(const FLOAT32 aArg) const
{
    return XFcVector2(x - aArg, y - aArg);
}


inline XFcVector2 & XFcVector2::operator*=(const INT32 aArg)
{
    x *= aArg;
    y *= aArg;
    return *this;
}


inline XFcVector2 & XFcVector2::operator*=(const FLOAT32 aArg)
{
    x *= aArg;
    y *= aArg;
    return *this;
}


inline XFcVector2 & XFcVector2::operator/=(const INT32 aArg)
{
    x /= aArg;
    y /= aArg;
    return *this;
}


inline XFcVector2 & XFcVector2::operator/=(const FLOAT32 aArg)
{
    x /= aArg;
    y /= aArg;
    return *this;
}


#ifndef XFC_NO_REAL_OPERATORS

inline XFcVector2 XFcVector2::operator+(const REAL &aArg) const
{
    return XFcVector2(x + aArg, y + aArg);
}

inline XFcVector2 XFcVector2::operator-(const REAL &aArg) const
{
    return XFcVector2(x - aArg, y - aArg);
}

inline XFcVector2 & XFcVector2::operator*=(const REAL &aArg)
{
    x *= aArg;
    y *= aArg;
    return *this;
}

inline XFcVector2 XFcVector2::operator*(const REAL &aArg) const
{
    return XFcVector2(x * aArg, y * aArg);
}

inline XFcVector2 & XFcVector2::operator/=(const REAL &aArg)
{
    x /= aArg;
    y /= aArg;
    return *this;
}

inline XFcVector2 XFcVector2::operator/(const REAL &aArg) const
{
    return XFcVector2(x / aArg, y / aArg);
}

inline XFcVector2 operator*(const REAL &aArg, const XFcVector2 &aVec)
{
    return aVec * aArg;
}


#else // or XFC_NO_REAL_OPERATORS


inline XFcVector2 XFcVector2::operator/(const FLOAT32 aArg) const
{
    return XFcVector2(x / aArg, y / aArg);
}


#endif //XFC_NO_REAL_OPERATORS


inline XFcVector2 XFcVector2::operator*(const INT32 aArg) const
{
    return XFcVector2(x * aArg, y * aArg);
}


inline XFcVector2 XFcVector2::operator*(const FLOAT32 aArg) const
{
    return XFcVector2(x * aArg, y * aArg);
}


inline XFcVector2 operator*(const INT32 aArg, const XFcVector2 &aVec)
{
    return aVec * aArg;
}


inline XFcVector2 operator*(const FLOAT32 aArg, const XFcVector2 &aVec)
{
    return aVec * aArg;
}


inline REAL XFcVector2::dotProduct(const XFcVector2 &aVec)
{
    return x * aVec.x + y * aVec.y;
}


inline REAL XFcVector2::length()
{
    //return (REAL)sqrt(lengthSq());
    return XFcMath::sqrt(lengthSq());
}


inline REAL XFcVector2::lengthSq()
{
    //float temp = (float)(x)*(float)(x) + (float)(y)*(float)(y) + (float)(z)*(float)(z);
    REAL temp = x * x + y * y;
    return REAL(temp);
}


inline void XFcVector2::normalize()
{
    REAL len = length();
    if (len != REALi(0))
        *this /= len;
/*
    float length = (float)sqrt(float(x) * float(x) +
                               float(y) * float(y) +
                               float(z) * float(z));
    if (length != 0)
    {
        *this /= length;
    }
*/
}


#endif // !XFC_MATH_VECTOR_H_INCLUDED
