/*
	File:			gzip.c

	Description:	Utility Functions

	Author:		

	Copyright:		Copyright  2003 by Tapwave, Inc.

	Disclaimer:		IMPORTANT:  This Tapwave software is provided by Tapwave, Inc. ("Tapwave").  Your 
					use is subject to and governed by terms and conditions of the Software Development 
					Kit Agreement ("SDK Agreement") between you and Tapwave.  If you have not entered 
					into Tapwaves standard SDK Agreement with Tapwave, you have no right or license 
					to use, reproduce, modify, distribute or otherwise exploit this Tapwave software.  
					You may obtain a copy of Tapwaves standard SDK Agreement by calling 650-960-1817
					or visiting Tapwave at http://www.tapwave.com/developers/. 

	Change History (most recent first):
				
*/

#include <TapWave.h>
#include "utils.h"

/*! \brief Decompress a gzip file stored in one memory chunk.  

\return A pointer to one memory chunk if successful, or else NULL.
*/

// From rfc1942.txt
typedef struct 
{
	UInt16   signature;
	UInt8    method;
	UInt8    flags;
	UInt32   datetime;
	UInt8    extraFlags;
	UInt8    OS;
} GZipFileHeader;

#define METHOD_DEFLATE 8

#define FTEXT    (1 << 0)
#define FHCRC    (1 << 1)
#define FEXTRA   (1 << 2)
#define FNAME 	 (1 << 3)
#define FCOMMENT (1 << 4)

int z_verbose;

void z_error(char *m)
{
}

extern int is_gzip(const void * sourceP, Int32 sourceSize)
{
	UInt16   signature = 0;
	
	if (sourceSize >= 2)
		pack("<h", (char*)&signature, (const char*)sourceP);
	
	return signature == 0x8b1f || signature == 0x1f8b;
}


extern void * gunzip(const void * sourceP, Int32 sourceSize)
{
	GZipFileHeader fileHeader;
	const void * p = sourceP;
	void * destP = NULL;
	UInt32 CRC16;
	UInt32 CRC32;
	UInt32 uncompressed_size;
	int result;
	z_stream stream;
	
	
	// First read the header.
	p = pack("<hbbibb", (char*)&fileHeader, (const char*)p);
	if (fileHeader.signature != 0x8b1f) 
		return NULL;
	
	if (fileHeader.flags & FEXTRA)
	{
		UInt16 length;
		
		// Ignore the subfields and just skip the whole thing.
		p = pack("<h", (char*)&length, (const char*)p);
	}
	
	// Skip the file name.  We could return a pointer to it though.
	if (fileHeader.flags & FNAME)
	{
		p = StrChr(p, '\0') + 1;
	}
	
	// Skip the comment.  We could return a pointer to it though.
	if (fileHeader.flags & FCOMMENT)
	{
		p = StrChr(p, '\0') + 1;
	}
	
	if (fileHeader.flags & FHCRC)
	{
		p = pack("<h", (char*)&CRC16, (const char*)p);
	}
	
	pack("<i", (char*)&CRC32, (const char*) sourceP + sourceSize - 8);
	pack("<i", (char*)&uncompressed_size, (const char*) sourceP + sourceSize - 4);

	// Now deflate the data.
	if (fileHeader.method != METHOD_DEFLATE)
		return NULL;
	
	// Allocate storage of the data.  The size is from the gzip format.
    destP = MemPtrNew(uncompressed_size);
	stream.next_out = (unsigned char *) destP;
	if (stream.next_out == NULL)
		return NULL;
	stream.avail_out = uncompressed_size;
	
	stream.next_in = (unsigned char *) p;
	stream.avail_in = sourceSize - 8 - ((char *) p - (char *) sourceP) + 1;
	stream.zalloc = Z_NULL;
	stream.zfree = Z_NULL;
	stream.opaque = Z_NULL;

	// Next is the data in deflate form, NOT in zlib form.  This means there
	// is no zlib header to read.
	//
	// The negative windowBits indicates that there is no zlib header.
	// In this case inflate requires there to be an extra byte after the
	// source, which is satified by gzip's crc32 bytes.
    result = inflateInit2(&stream, -MAX_WBITS);
    
    // Calling inflate once should result in all data being inflated, because we
    // read it all in and made it all available.  But loop until inflate() 
    // returns that it has reached the end, just in case.
	while (result == Z_OK && stream.avail_in > 0)
		result = inflate(&stream, Z_FINISH);
	if (result == Z_STREAM_END)
		result = inflateEnd(&stream);
	
	// We want to return NULL if there is an error.  A non NULL pointer should only
	// be returned on sucess.
	if (result != Z_OK && destP != NULL)
	{
		MemPtrFree(destP);
		destP = NULL;
	}
		
	return destP;
}
